package api

import (
	"bufio"
	"bytes"
	"fmt"
	"io"
	"io/ioutil"
	"net/http"
	"os"
	"os/exec"
	"path"
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
	"strings"
	"time"

	"github.com/gin-gonic/gin"
)

func CmdExe(commandName string, params []string) error {

	cmd := exec.Command(commandName, params...)
	stdout, err := cmd.StdoutPipe()
	if err != nil {
		fmt.Println("cmd.StdoutPipe: ", err)
		return err
	}

	var errOut bytes.Buffer
	cmd.Stderr = &errOut

	//创建一个流来读取管道内内容，这里逻辑是通过一行一行的读取的
	reader := bufio.NewReader(stdout)
	//实时循环读取输出流中的一行内容
	go func() {
		for {
			line, err2 := reader.ReadString('\n')
			if err2 != nil || io.EOF == err2 {
				break
			}
			fmt.Println("cmd=》", line)
		}
	}()

	err = cmd.Start()
	if err != nil {
		return err
	}

	return cmd.Wait()
}

func PathExists(path string) (bool, error) {

	_, err := os.Stat(path)
	if err == nil {
		return true, nil
	}
	if os.IsNotExist(err) {
		return false, nil
	}
	return false, err
}

//ServiceTestHttp JWT授权的http处理函数
func ServiceConvModel(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	body := struct {
		File   *model.OssType
		DocId  string
		MeshId string
	}{}
	err := c.ShouldBindJSON(&body)
	if err != nil {
		return nil, err
	}

	if len(body.DocId) < 1 {
		return nil, NewError("模型Id不能为空")
	}

	if body.File == nil || len(body.File.Url) < 1 {
		return nil, NewError("模型链接不能为空")
	}

	sDir, modelName := path.Split(body.File.Url)

	wkspace := fmt.Sprintf("%d", time.Now().UnixNano())
	dir := fmt.Sprintf("uploads/%s", wkspace)
	err = os.MkdirAll(dir, 0777)

	if err != nil {
		return nil, NewError("创建目录失败!")
	}

	//1下载文件到本地
	modelFile := fmt.Sprintf("%s/%s", dir, modelName)
	out, err := os.Create(modelFile)
	if err != nil {
		os.RemoveAll(dir)
		return nil, err
	}

	meshUrl := body.File.Url
	if !strings.HasPrefix(meshUrl, "http") {
		meshUrl = fmt.Sprintf("https:%s", body.File.Url)
	}
	resp, err := http.Get(meshUrl)
	if err != nil {
		os.RemoveAll(dir)
		return nil, err
	}
	defer resp.Body.Close()

	pix, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		os.RemoveAll(dir)
		return nil, err
	}
	_, err = io.Copy(out, bytes.NewReader(pix))
	if err != nil {
		os.RemoveAll(dir)
		return nil, err
	}
	out.Close()

	// //调用转换器进行转化
	// //2 文件转.osg.gles

	relativePath := fmt.Sprintf("%s/%s", wkspace, modelName)

	err = CmdExe("docker", []string{"exec", "osg", "osgconv", "--use-world-frame", "-O", "useExternalBinaryArray mergeAllBinaryFiles", "-o", "0,1,0-0,0,1", fmt.Sprintf(`/data/%s`, relativePath), fmt.Sprintf(`/data/%s.osg.gles`, relativePath)})
	if err != nil { //获取输出对象，可以从该对象中读取输出结果
		os.RemoveAll(dir)
		return nil, err
	}

	//3 osg.gles 转 .osgjs

	err = CmdExe("docker", []string{"exec", "osg", "osgconv", "-O", "useExternalBinaryArray mergeAllBinaryFiles disableStrictJson", fmt.Sprintf(`/data/%s.osg.gles`, relativePath), fmt.Sprintf(`/data/%s.osgjs`, relativePath)})
	if err != nil {
		os.RemoveAll(dir)
		return nil, err
	}

	//resultFile = { name: modelName, main: path.join(dir, `${modelName}.osgjs`), assets: [path.join(dir, `${modelName}.bin`)] }

	osgjs := fmt.Sprintf(`%s/%s.osgjs`, dir, modelName)
	osgjsbin := fmt.Sprintf(`%s/%s.bin`, dir, modelName)

	fmt.Sprintln(osgjs, osgjsbin)

	ext, _ := PathExists(osgjs)
	if !ext {
		os.RemoveAll(dir)
		return nil, NewError("文件转化失败!")
	}

	ext, _ = PathExists(osgjsbin)
	if !ext {
		os.RemoveAll(dir)
		return nil, NewError("文件转化失败!")
	}

	//上传文件到oss的同级目录 ${dir}/result/*.gz
	resultJs, resultBin, err := UploadObsConvAssets(apictx, sDir, osgjs, osgjsbin)
	if err != nil {
		os.RemoveAll(dir)
		return nil, err
	}
	mesh := &model.Mesh{File: body.File, Osgjsbin: resultBin, Osgjs: resultJs}
	saved, err := repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionMesh, body.MeshId, mesh)
	if err != nil {
		os.RemoveAll(dir)
		return nil, err
	}

	outresult := map[string]interface{}{
		"osgjs": resultJs,
		"saved": saved,
	}
	os.RemoveAll(dir)

	return outresult, nil
}

func UploadObsConvAssets(apictx *ApiSession, filePath string, osgjsFilePath string, osgbinFilePath string) (*model.OssType, *model.OssType, error) {
	seg := strings.Split(filePath, "myhuaweicloud.com/")
	ossPath := seg[len(seg)-1]

	client, err := CreateObsClient()
	if err != nil {
		return nil, nil, NewError("创建ObsClient 失败!")
	}

	defer func() {
		client.Close()
	}()

	bucketName := apictx.Svc.Conf.Obs.Bucket
	resultJs := UploadFile(client, bucketName, osgjsFilePath, ossPath)
	if resultJs.Size < 1 {
		return nil, nil, NewError("上传osg js失败!")
	}
	resultBin := UploadFile(client, bucketName, osgbinFilePath, ossPath)
	if resultBin.Size < 1 {
		return nil, nil, NewError("上传osg bin失败!")
	}
	return resultJs, resultBin, nil
}

func UploadMinioConvAssets(apictx *ApiSession, filePath string, osgjsFilePath string, osgbinFilePath string) (*model.OssType, *model.OssType, error) {

	bucketName := apictx.Svc.Conf.Minio.Bucket

	sepStr := fmt.Sprintf("com/minio/%s/", bucketName)

	seg := strings.Split(filePath, sepStr)
	ossPath := seg[len(seg)-1]

	client := createMinioClient(apictx.Svc.Conf)

	resultJs, jsSize := MinioUploadLocalFile(client, bucketName, osgjsFilePath, ossPath)
	if len(resultJs) < 1 {
		return nil, nil, NewError("上传osg js失败!")
	}

	resultBin, binSize := MinioUploadLocalFile(client, bucketName, osgbinFilePath, ossPath)
	if len(resultBin) < 1 {
		return nil, nil, NewError("上传osg bin失败!")
	}

	return &model.OssType{
			Url:  fmt.Sprintf("%s/%s/%s", apictx.Svc.Conf.Minio.PubHost, bucketName, resultJs),
			Size: jsSize,
		},
		&model.OssType{
			Url:  fmt.Sprintf("%s/%s/%s", apictx.Svc.Conf.Minio.PubHost, bucketName, resultBin),
			Size: binSize,
		}, nil
}

func ConvMeshFile(file *model.OssType, apictx *ApiSession) (*model.OssType, *model.OssType, error) {

	sDir, modelName := path.Split(file.Url)

	wkspace := fmt.Sprintf("%d", time.Now().UnixNano())
	dir := fmt.Sprintf("uploads/%s", wkspace)
	err := os.MkdirAll(dir, 0777)

	if err != nil {
		return nil, nil, NewError("创建目录失败!")
	}

	//1下载文件到本地
	modelFile := fmt.Sprintf("%s/%s", dir, modelName)
	out, err := os.Create(modelFile)
	if err != nil {
		os.RemoveAll(dir)
		return nil, nil, err
	}

	meshUrl := file.Url
	if !strings.HasPrefix(meshUrl, "http") {
		meshUrl = fmt.Sprintf("https:%s", file.Url)
	}
	resp, err := http.Get(meshUrl)
	if err != nil {
		os.RemoveAll(dir)
		return nil, nil, err
	}
	defer resp.Body.Close()

	pix, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		os.RemoveAll(dir)
		return nil, nil, err
	}
	_, err = io.Copy(out, bytes.NewReader(pix))
	if err != nil {
		os.RemoveAll(dir)
		return nil, nil, err
	}
	out.Close()

	// //调用转换器进行转化
	// //2 文件转.osg.gles

	relativePath := fmt.Sprintf("%s/%s", wkspace, modelName)

	err = CmdExe("docker", []string{"exec", "osg", "osgconv", "--use-world-frame", "-O", "useExternalBinaryArray mergeAllBinaryFiles", "-o", "0,1,0-0,0,1", fmt.Sprintf(`/data/%s`, relativePath), fmt.Sprintf(`/data/%s.osg.gles`, relativePath)})
	if err != nil { //获取输出对象，可以从该对象中读取输出结果
		os.RemoveAll(dir)
		return nil, nil, err
	}

	//3 osg.gles 转 .osgjs

	err = CmdExe("docker", []string{"exec", "osg", "osgconv", "-O", "useExternalBinaryArray mergeAllBinaryFiles disableStrictJson", fmt.Sprintf(`/data/%s.osg.gles`, relativePath), fmt.Sprintf(`/data/%s.osgjs`, relativePath)})
	if err != nil {
		os.RemoveAll(dir)
		return nil, nil, err
	}
	osgjs := fmt.Sprintf(`%s/%s.osgjs`, dir, modelName)
	osgjsbin := fmt.Sprintf(`%s/%s.bin`, dir, modelName)

	fmt.Sprintln(osgjs, osgjsbin)

	ext, _ := PathExists(osgjs)
	if !ext {
		os.RemoveAll(dir)
		return nil, nil, NewError("文件转化失败!")
	}

	ext, _ = PathExists(osgjsbin)
	if !ext {
		os.RemoveAll(dir)
		return nil, nil, NewError("文件转化失败!")
	}

	//上传文件到oss的同级目录 ${dir}/result/*.gz
	resultJs, resultBin, err := UploadObsConvAssets(apictx, sDir, osgjs, osgjsbin)
	if err != nil {
		os.RemoveAll(dir)
		return nil, nil, err
	}

	os.RemoveAll(dir)
	return resultJs, resultBin, nil
}
