package api

import (
	"context"
	"fmt"
	"sku3dweb/db/model"
	"sku3dweb/db/repo"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func ServiceUserLoginPassword(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	body := struct {
		Phone    string
		Password string
	}{}
	err := c.ShouldBindJSON(&body)
	if err != nil {
		return nil, err
	}

	if len(body.Phone) < 1 {
		return nil, NewError("手机号码不能为空")
	}

	pwd := ""
	if len(body.Password) > 0 {
		pwd = UtilMd5(body.Password)
	}
	fmt.Println(pwd)

	// ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
	repoCtx := apictx.CreateRepoCtx()

	user := &model.User{}
	options := &repo.DocSearchOptions{Query: repo.Map{"phone": body.Phone, "password": pwd}, CollectName: repo.CollectionUser}
	finded, err := repo.RepoSeachDoc(repoCtx, options, user)

	if err != nil { //主账号登录，没有登录成功，使用子账号登录
		return nil, err
	}
	if !finded {
		return nil, NewError("账号密码不对")
	}

	if user.State == -1 {
		return nil, NewError("当前用户已禁用,请使用规定的账号重新登录!")
	}

	repo.UserLoginSucc(repoCtx, user.Id, user.Phone)

	jwtU := &JWTUser{ID: user.GetID(), Parent: user.Parent, Phone: user.Phone, State: 1,Key: "mall-merchant"}
	if len(jwtU.Parent) < 1 {
		jwtU.Parent = jwtU.ID
	}
	token, _, err := apictx.Svc.JWT.JwtCreateToken(jwtU)

	if err != nil {
		return nil, err
	}

	out := map[string]interface{}{
		"token":         token,
		"user":          user,
		"isMainAccount": jwtU.Parent == jwtU.ID,
	}

	return out, nil
}

func ServiceUserLoginSms(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	body := struct {
		Phone string
		Code  string
	}{}
	err := c.ShouldBindJSON(&body)
	if err != nil {
		return nil, NewError("解析参数错误!")
	}
	if len(body.Phone) != 11 || len(body.Code) < 1 {
		return nil, NewError("参数错误!")
	}

	cacheKey := fmt.Sprintf("sms#%s#login", body.Phone)

	rdb := apictx.Svc.Redis
	ctx := context.Background()

	// cacheCode, _ := rdb.Get(ctx, cacheKey).Result()
	// if len(cacheCode) < 1 {
	// 	 return nil, NewError("验证码已过期!")
	// }

	// if cacheCode != body.Code {
	// 	 return nil, NewError("验证码错误!")
	// }

	repCtx := apictx.CreateRepoCtx()

	user, err := repo.UserSearchByPhone(repCtx, body.Phone)
	if err != nil {
		return nil, NewError("查询用户失败!")
	}

	isNew := false
	if user == nil { //新用户，创建新用户

		user = &model.User{Name: body.Phone, Phone: body.Phone, Avatar: "https://os.alipayobjects.com/rmsportal/QBnOOoLaAfKPirc.png",
			UserType: IntValue(model.USERTYPE_PERSONAL)}
		newUid := repo.AddUser(repCtx, user)
		if len(newUid) < 1 {
			return nil, NewLogError("创建用户失败")
		}
		user.Id, _ = primitive.ObjectIDFromHex(newUid)
		isNew = true
	}

	//删除验证码
	rdb.Del(ctx, cacheKey)

	// parents := repo.GetUserParents(repCtx, body.Phone)

	repo.UserLoginSucc(repCtx, user.Id, body.Phone)

	token, _, err := apictx.Svc.JWT.JwtCreateToken(&JWTUser{ID: user.GetID(), Parent: user.GetID(), Phone: user.Phone, State: 1,Key: "mall-merchant"})

	if err != nil {
		return nil, err
	}

	out := map[string]interface{}{
		"token": token,
		"user":  user,
		// "parent":        parents,
		"isMainAccount": true,
		"isNew":         isNew,
	}

	return out, nil
}

func ServiceLoginSwitchParent(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	body := struct {
		ParentId string `json:"_id"`
	}{}
	err := c.ShouldBindJSON(&body)
	if err != nil {
		return nil, NewError("解析参数错误!")
	}
	if len(body.ParentId) < 1 {
		return nil, NewError("参数错误!")
	}

	token, _, err := apictx.Svc.JWT.JwtCreateToken(&JWTUser{ID: apictx.User.ID, Parent: body.ParentId, Phone: apictx.User.Phone, State: 1})
	if err != nil {
		return nil, err
	}

	out := map[string]interface{}{
		"token": token,
	}
	return out, nil
}

func ServiceLoginAlipay(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	return nil, nil
}

func ServiceLoginWechat(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	return nil, nil
}

func ServiceWebUserProfile(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	user := apictx.User

	u, err := repo.UserSearchById(apictx.CreateRepoCtx(), user.ID)
	if err != nil {
		return nil, err
	}

	if u == nil {
		return nil, NewErrorWithCode("登录异常!", 401)
	}

	isMainAccount := user.Parent == user.ID
	out := map[string]interface{}{
		"user":          u,
		"isMainAccount": isMainAccount,
		"perms":         []string{},
		"team":          "",
		"isTeamLeader":  false,
		"isPlatform":    user.Parent == PLATFORM_USER_ID,
	}
	//是否是平台账号

	//查询所有权限

	if len(u.Roles) > 0 {
		roleIds := []primitive.ObjectID{}
		for _, r := range u.Roles {
			rid, _ := primitive.ObjectIDFromHex(r)
			roleIds = append(roleIds, rid)
		}
		ok, perms := repo.RepoSeachDocsMap(apictx.CreateRepoCtx(),
			&repo.DocsSearchOptions{CollectName: repo.CollectionRoles,
				Query:   repo.Map{"_id": bson.M{"$in": roleIds}},
				Project: []string{"name", "perms"},
			})
		if ok {
			out["perms"] = perms
		}
	}

	if isMainAccount && *u.UserType == model.USERTYPE_COMPANY { //企业主账号返回所有权限
		ok, perms := repo.RepoSeachDocsMap(apictx.CreateRepoCtx(),
			&repo.DocsSearchOptions{CollectName: repo.CollectionRoles,
				Project: []string{"name", "perms"},
			})
		if ok {
			out["perms"] = perms
		}
	}

	//查询团对信息
	if len(u.TeamId) > 0 {
		ok, team := repo.RepoSeachDocMap(apictx.CreateRepoCtx(),
			&repo.DocSearchOptions{CollectName: repo.CollectionTeam,
				Query:   repo.Map{"_id": u.TeamId},
				Project: []string{"name"},
			})

		if ok {
			out["team"] = team
		}
	}

	//是否是队长
	ok, _ := repo.RepoSeachDocMap(apictx.CreateRepoCtx(),
		&repo.DocSearchOptions{CollectName: repo.CollectionTeam,
			Query:   repo.Map{"_id": u.TeamId, "leaderId": user.ID},
			Project: []string{"name"},
		})
	if ok {
		out["isTeamLeader"] = true
	}
	return out, nil
}

func ServiceWebUserUpdateProfile(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	body := struct {
		Avatar  string
		City    string
		Company string
		Desc    string
		Name    string
	}{}
	err := c.ShouldBindJSON(&body)
	if err != nil {
		return nil, NewError("解析参数错误!")
	}
	user := &model.User{Avatar: body.Avatar, City: body.City, Company: body.Company, Desc: body.Desc, Name: body.Name}

	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionUser, apictx.User.ID, user)
}

func ServiceWebUserResetPwd(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	body := struct {
		Password string
	}{}
	err := c.ShouldBindJSON(&body)
	if err != nil {
		return nil, NewError("解析参数错误!")
	}
	if len(body.Password) < 1 {
		return nil, NewError("密码不能为空!")
	}

	body.Password = UtilMd5(body.Password)
	user := &model.User{Password: body.Password}
	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionUser, apictx.User.ID, user)
}

func ServiceWebUserUpdateEditSet(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	id := apictx.User.ID
	theme := model.EnvTheme{}
	err := c.ShouldBindJSON(&theme)
	if err != nil {
		return nil, err
	}

	if len(theme.Id) < 1 {
		return nil, NewError("参数Id不能为空!")
	}

	//查询对应的环境是否已经存在
	u := &model.User{}

	ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		CollectName: repo.CollectionUser,
		Query:       repo.Map{"_id": id}, //"editorSet.themes": bson.M{"$elemMatch": bson.M{"id": theme.Id}}}
		Project:     []string{"editorSet.themes"},
	}, u)

	if err != nil {
		return nil, err
	}

	if !ok {
		return nil, NewError("登录异常！请重新登录！")
	}

	if u.EditorSet == nil || u.EditorSet.Themes == nil {

		u.Id = primitive.NilObjectID
		if u.EditorSet == nil {
			u.EditorSet = &model.EditorSetting{
				Themes: []*model.EnvTheme{
					&theme,
				},
			}
		} else {
			u.EditorSet.Themes = []*model.EnvTheme{
				&theme,
			}
		}

		return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionUser, id, u)
	}

	finded := false
	for _, v := range u.EditorSet.Themes {
		if v.Id == theme.Id {
			finded = true
			break
		}
	}

	if finded {
		option := &repo.ArrayOneUpdateOption{
			CollectName: repo.CollectionUser,
			Id:          id,
			Query:       repo.Map{"editorSet.themes.id": theme.Id},
			Set:         repo.Map{"editorSet.themes.$": theme},
		}
		return repo.RepoDocArrayOneUpdate(apictx.CreateRepoCtx(), option)
	}

	return repo.RepoDocArrayAppend(apictx.CreateRepoCtx(), repo.CollectionUser, id, "editorSet.themes", theme)
}

func ServiceWebUserUpdateEditSetBaseColor(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	id := apictx.User.ID
	body := struct {
		Color model.Vect3
	}{}

	err := c.ShouldBindJSON(&body)
	if err != nil {
		return nil, err
	}
	//查询对应的环境是否已经存在
	return repo.RepoUpdateSetDocProps(apictx.CreateRepoCtx(), repo.CollectionUser, id, bson.M{"$set": bson.M{"editorSet.modelBaseColor": body.Color}})
}

func ServiceWebUserUpdateEditSetDefaultEnv3d(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	body := struct {
		Id string
	}{}
	err := c.ShouldBindJSON(&body)
	if err != nil {
		return nil, err
	}

	//查询对应的环境是否已经存在
	return repo.RepoUpdateSetDocProps(apictx.CreateRepoCtx(), repo.CollectionUser, apictx.User.ID, bson.M{"$set": bson.M{"editorSet.defaultEnv3dId": body.Id}})
}

func ServiceWebUserLoginout(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	return true, nil
}
