package main

import (
	"flag"
	"fmt"
	"sku3dweb/api"
	"sku3dweb/bus"
	"sku3dweb/conf"
	"sku3dweb/db"
	"sku3dweb/log"

	"sku3dweb/comm"

	"go.uber.org/dig"
)

var confFile = flag.String("conf", "./app.yaml", "conf file")

func BuildApp() *dig.Container {

	app := dig.New()
	_ = app.Provide(func() (*conf.AppConf, error) {
		return conf.NewAppConf(*confFile)
	})
	_ = app.Provide(bus.NewNatsBus)
	_ = app.Provide(db.NewMongoDB)
	_ = app.Provide(db.NewRedisClient)
	_ = app.Provide(api.NewHttpService)

	return app
}

func main() {

	fmt.Println(api.UtilMd5("123456"))

	flag.Parse()
	app := BuildApp()

	err := app.Invoke(func(config *conf.AppConf, busCenter *comm.NatsBus, svc *api.Service) error {
		go busCenter.Run(nil)

		svc.Run()
		return nil
	})

	if err != nil {
		log.Errorf("the pink service start fail %+v", err)
		panic(err)
	}
}
